/**
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * Copyright (C) 2013, 2014 Department of Informatics and
 * Telecommunications, National and Kapodistrian University of Athens
 *
 *  Created on: Apr 25, 2014
 *      Author: Charalampos Nikolaou <charnik@di.uoa.gr>
 */
#ifndef GMLFORMATWRITER_H_
#define GMLFORMATWRITER_H_

#include <fstream>
#include "NetworkFileFormatWriter.h"

class GMLFormatWriter: public NetworkFileFormatWriter {
public:
	GMLFormatWriter() { }

	virtual std::string get_extension() {
		return "gml";
	}

	virtual void write(const std::string &network_name, const PartGraph *pgraph) {
		std::ofstream ofile(get_filename(network_name, pgraph).c_str());

		ofile << "graph [\n";
		ofile << "  label \"" << get_graph_name(network_name, pgraph) << "\"\n";

		// list the nodes of the partitioning graph
		for (partition_map::const_iterator it = pgraph->partitions_begin();
				                                 it != pgraph->partitions_end(); ++it) {
			ofile << "  node [\n";
			ofile << "    id " << it->first << "\n";
			ofile << "    label \"" << it->first << "\"\n";
			ofile << "  ]\n";
		}

		// list the edges of the partitioning graph
		const edge_list& part_edges = pgraph->get_edges();
		edge_list::const_iterator it;
		for (it = part_edges.begin(); it != part_edges.end(); ++it) {
			ofile << "  edge [\n";
			ofile << "    source " << it->first << "\n";
			ofile << "    target " << it->second << "\n";
			ofile << "  ]\n";
		}

		ofile << "]\n";
		ofile.close();
	}

	virtual ~GMLFormatWriter() { }
};

#endif /* GMLFORMATWRITER_H_ */
