/**
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * Copyright (C) 2013, 2014 Department of Informatics and
 * Telecommunications, National and Kapodistrian University of Athens
 *
 *  Created on: Jan 16, 2013
 *      Author: Charalampos Nikolaou <charnik@di.uoa.gr>
 */
#include "ReasoningService.h"

bool ReasoningService::check_consistency() {
	bool status;
	bool repartition_request = opts.repartition;
	bool repartitioned;

	do {
		repartitioned = false;

		long int stime = util::get_time();
		PartGraph *pgraph = picker.pick_best_partition();
		network.part_desc = std::string(pgraph->get_desc());

		if (!opts.quiet) pgraph->print_stats();

		ConsistencyEngine engine(pgraph);

		// && !(opts.threshold && pgraph->need_more_partitions())) {
		status = engine.execute(repartition_request | opts.pconly);

		if (status && repartition_request) {
			engine.update_metis_graph();

			picker.initialize();

			//picker.compute_partitions();
			if (!partition()) { // re-partition
				std::cerr << "error while re-partitioning the network\n";
			}

			repartitioned = true;
			repartition_request = false;
		}
		long int etime = util::get_time();

		network.add_to_consistency_duration(etime - stime);

		// check sanity of the result (but do not include it in the measured time)
		if (status && opts.sanity_check && !repartitioned) { // check only in the final step
			engine.sanity_check(status);
		}
	} while (repartitioned);

	network.consistent = status;

	return status;
}

bool ReasoningService::check_traditional_consistency() const {
	bool status = false;
	network.part_desc = "-";

	long int stime = util::get_time();
	MetisGraph mgraph(network.filename);
	if (mgraph.read()) {
		RCC8_CSP csp = RCC8_CSP(mgraph.get_graph_nodes(),
								mgraph.get_graph_edges(),
								mgraph.get_xadj(),
								mgraph.get_adjncy(),
								mgraph.get_adjrel());
		if (opts.pconly) {
			status = csp.path_consistent();

		} else {
			status = csp.consistent();
		}
	}
	long int etime = util::get_time();

	network.add_to_consistency_duration(etime - stime);
	network.consistent = status;

	return status;
}

void ReasoningService::print_pgraph_to_file() {
	NetworkFileFormatWriter *writer = NetworkFileFormatWriterFactory::create_writer(opts.pgraph_format);
	writer->write(network.name, picker.pick_best_partition());
	FREE_PTR(writer);
}

