/**
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * Copyright (C) 2013, 2014 Department of Informatics and
 * Telecommunications, National and Kapodistrian University of Athens
 *
 *  Created on: Jan 16, 2013
 *      Author: Charalampos Nikolaou <charnik@di.uoa.gr>
 */
#ifndef REASONINGSERVICE_H_
#define REASONINGSERVICE_H_

#include "prcc8.h"
#include "PartGraphPicker.h"
#include "ConsistencyEngine.h"
#include "rcc8csp.h"
#include "NetworkFileFormatWriterFactory.h"

class ReasoningService {
private:
	graph_info &network;
	const cmd_options &opts;

	PartGraphPicker picker;

public:
	ReasoningService(graph_info &graph, const cmd_options &options):
		network(graph), opts(options), picker(PartGraphPicker(network.filename)) {
	}

	bool partition() {
		bool status;

		long int stime = util::get_time();
		if (opts.nparts > 1) { // run a specific scenario
			status = picker.partition(opts.nparts, opts.strategy, opts.obj_volume);

		} else { // nparts = 0; compute all possible scenaria and execute the best one
			status = picker.compute_partitions();
		}
		long int etime = util::get_time();

		network.add_to_partitioning_time(etime - stime);

		return status;
	}

	bool check_traditional_consistency() const;
	bool check_consistency();

	inline void estimate_memory_requirements() const {
		MetisGraph mgraph = MetisGraph(network.filename);

		if (mgraph.read(true)) {
			idx_t nodes = mgraph.num_nodes();

			long mem = RCC8_CSP::memory_needs(nodes/opts.nparts);

			mem *= opts.nparts;

			std::cout << "\nMemory requirements\n";
			std::cout << "----------------------\n";
			std::cout << "Number of nodes: " << mgraph.num_nodes() << "\n";
			std::cout << "Number of edges: " << mgraph.num_edges() << "\n";
			std::cout << "Nodes/partition: " << nodes/opts.nparts << " (perfect partitioning)\n";
			std::cout << "Memory needed  : " << Util::get_human_size(mem) << "\n";
		}
	}

	void print_pgraph_to_file();

	~ReasoningService() {}

	inline static void init_env() {
		RCC8_CSP::initfop();
	}
};

#endif /* REASONINGSERVICE_H_ */
