/**
s * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * Copyright (C) 2013, 2014 Department of Informatics and
 * Telecommunications, National and Kapodistrian University of Athens
 *
 *  Created on: Apr 24, 2014
 *      Author: Charalampos Nikolaou <charnik@di.uoa.gr>
 */
#ifndef SIFFORMATWRITER_H_
#define SIFFORMATWRITER_H_

#include <fstream>
#include "NetworkFileFormatWriter.h"

class SIFFormatWriter : public NetworkFileFormatWriter {
public:
	SIFFormatWriter() {	}

	virtual std::string get_extension() {
		return "sif";
	}

	virtual void write(const std::string &network_name, const PartGraph *pgraph) {
		std::ofstream ofile(get_filename(network_name, pgraph).c_str());

		// iterate over the edges of the partitioning graph
		const edge_list& part_edges = pgraph->get_edges();
		edge_list::const_iterator it;
		for (it = part_edges.begin(); it != part_edges.end(); ++it) {
			// ... and list them
			ofile << it->first << " edge " << it->second << "\n";
		}

		ofile.close();
	}

	virtual ~SIFFormatWriter() { }
};

#endif /* SIFFORMATWRITER_H_ */
