/**
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * Copyright (C) 2013, 2014 Department of Informatics and
 * Telecommunications, National and Kapodistrian University of Athens
 *
 *  Created on: Dec 19, 2013
 *      Author: Charalampos Nikolaou <charnik@di.uoa.gr>
 */
#ifndef XGMMLFORMATWRITER_H_
#define XGMMLFORMATWRITER_H_

#include <fstream>
#include "NetworkFileFormatWriter.h"

class XGMMLFormatWriter : public NetworkFileFormatWriter {
public:
	XGMMLFormatWriter() { }

	virtual std::string get_extension() {
		return "xgmml";
	}

	virtual void write(const std::string &network_name, const PartGraph *pgraph) {
		std::ofstream ofile(get_filename(network_name, pgraph).c_str());

		ofile << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
		ofile << "<graph label=\"" << get_graph_name(network_name, pgraph) << "\" directed=\"0\" xmlns=\"http://www.cs.rpi.edu/XGMML\">\n";

		// list the nodes of the partitioning graph
		for (partition_map::const_iterator it = pgraph->partitions_begin();
				                                 it != pgraph->partitions_end(); ++it) {
			ofile << "  <node id=\"" << it->first << "\" label=\"" << it->first << "\">\n";
			ofile << "    <att name=\"size\" type=\"integer\" value=\"" << it->second.size() << "\"/>\n";
			ofile << "  </node>\n";
		}

		// list the edges of the partitioning graph
		const edge_list& part_edges = pgraph->get_edges();
		edge_list::const_iterator it;
		for (it = part_edges.begin(); it != part_edges.end(); ++it) {
			ofile << "  <edge source=\"" << it->first << "\" target=\"" << it->second << "\">\n";
			ofile << "    <att name=\"weight\" type=\"integer\" value=\"" <<
								pgraph->get_common_edges(it->first, it->second).size() << "\"/>\n";
			ofile << "  </edge>\n";

		}

		ofile << "</graph>\n";
		ofile.close();
	}

	virtual ~XGMMLFormatWriter() { }
};

#endif /* XGMMLFORMATWRITER_H_ */
