/**
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * Copyright (C) 2013, 2014 Department of Informatics and
 * Telecommunications, National and Kapodistrian University of Athens
 *
 *  Created on: Jan 4, 2014
 *      Author: Charalampos Nikolaou <charnik@di.uoa.gr>
 */
#ifndef DATASTRUCT_H_
#define DATASTRUCT_H_

#include <set>
#include <list>
#include <boost/unordered_map.hpp>
#include <boost/dynamic_bitset.hpp>


/* a bitset of fixed size number of bits */
typedef boost::dynamic_bitset<> bit_set;

typedef std::pair<idx_t, idx_t> edge_t;

/* a list of edges (pairs of ids) */
typedef std::list<edge_t> edge_list;

/* a set of edges (pairs of ids) */
typedef std::set<edge_t> edge_set;

/* map from edges (edgecut) to list of edges */
typedef boost::unordered_map<edge_t, edge_list> edgecut_map;

/* map from edges (partitioning graph edge) to list of edges (shared edges between two partitions) */
typedef boost::unordered_map<edge_t, edge_list> dependency_map;

#endif /* DATASTRUCT_H_ */
