/**
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * Copyright (C) 2013, 2014 Department of Informatics and
 * Telecommunications, National and Kapodistrian University of Athens
 *
 *  Created on: Dec 19, 2013
 *      Author: Charalampos Nikolaou <charnik@di.uoa.gr>
 */
#ifndef P_STRAT_H_
#define P_STRAT_H_

#include <iostream>
#include <vector>

#include <boost/program_options/errors.hpp>

#include "MetisGraph.h"
#include "PartGraph.h"
#include "rcc8.h"

#define BALANCE_NAME			"balanced"
#define BLINKERED_NAME			"blinkered"
#define MIN_FRINGE_NAME 		"min_fringe"
#define MIN_INTERSECTION_NAME	"min_inter"


/* declaration of partition strategies */
/* PSTRAT_BEGIN and PSTRAT_END values may be used as iterators */
enum p_strat {PSTRAT_BEGIN, BALANCED = PSTRAT_BEGIN, BLINKERED, MIN_FRINGE, MIN_INTERSECTION, PSTRAT_END};

/* functions for implementing the various partition strategies */
void pg_create_balanced(MetisGraph &mgraph, PartGraph *pgraph);
void pg_create_blinkered(MetisGraph &mgraph, PartGraph *pgraph);
void pg_create_minfringe(MetisGraph &mgraph, PartGraph *pgraph);
void pg_create_mininter(MetisGraph &mgraph, PartGraph *pgraph);

// code shared among the strategies and executed at the beginning of the partitioning
void pg_pre_create(MetisGraph &mgraph, PartGraph *pgraph, long *start);

// code shared among the strategies and executed at the end of the partitioning
void pg_post_create(MetisGraph &mgraph, PartGraph *pgraph, const edge_set &cut, long start);

std::istream& operator>>(std::istream& in, p_strat& strat);
p_strat& operator++(p_strat& strat);
p_strat operator++(p_strat& strat, int);
p_strat& operator--(p_strat& strat);
p_strat operator--(p_strat& strat, int);

// see file p_strat.h
extern const char *p_strat_alias[];

// see file p_strat.h
extern void (*p_strat_impl[])(MetisGraph &, PartGraph *);

/**
 * Get the alias for a partitioning strategy.
 *
 * @param strategy a partitioning strategy
 * @return the alias for the partition strategy given in \a strategy
 */
inline const char* strategy_alias(p_strat strategy) {
	return p_strat_alias[strategy];
}

/**
 * Checks whether a partitioning strategy value is valid one.
 *
 * @param strategy the partitioning strategy
 * @return true in case it is valid, false otherwise
 */
inline bool valid_strategy(p_strat strategy) {
	return (strategy >= PSTRAT_BEGIN && strategy < PSTRAT_END);
}

#endif /* P_STRAT_H_ */
