/**
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * Copyright (C) 2013, 2014 Department of Informatics and
 * Telecommunications, National and Kapodistrian University of Athens
 *
 *  Created on: Jan 15, 2013
 *      Author: Charalampos Nikolaou <charnik@di.uoa.gr>
 */
#ifndef PRCC8_H_
#define PRCC8_H_

#define PROG_NAME "gp-rcc8"
#define GEN_NAME "plaw-gen"

#define PRCC8_VERSION "gp-rcc8-0.2-alpha"
#define PGEN_VERSION "plaw-gen-0.1-alpha"
#define LICENSE	"Copyright (C) 2013, 2014 Department of Informatics and Telecommunications,\n\
National and Kapodistrian University of Athens.\n\
This program is free software. You can redistribute it and/or modify it\n\
under the terms of the GNU General Public License."

#include <string>
#include "p_strat.h"
#include "netviz_format.h"

/* hold information about each input network */
typedef struct graph_info {
	const char *filename;		/*< filename of the network as given in the command line */
	std::string name;			/*< basename of the network */
	bool consistent;			/*< the result of the execution of consistency */
	std::string part_desc;		/*< the descriptor identifying how the network was partitioned */

	// timing statistics
	long int partitioning_time;
	long int consistency_time;

	explicit graph_info(const char *file) {
		filename = file;
		consistent = false;

		// timing
		partitioning_time = 0;
		consistency_time = 0;

		// set the name of the network
		name = std::string(filename);

		size_t slash_pos = name.rfind('/') + 1; // go past last '/' character
		size_t dot_pos = name.rfind('.');
		if (dot_pos == string::npos) {
			name = name.substr(slash_pos);

		} else {
			name = name.substr(slash_pos, dot_pos - slash_pos);
		}

		// debug
		//std::cout << "Name: " << name << "\n";
	}

	inline void add_to_consistency_duration(long int duration) {
		consistency_time += duration;
	}

	inline void add_to_partitioning_time(long int duration) {
		partitioning_time += duration;
	}
} graph_info;

/* hold command line options */
typedef struct cmd_options {
	int nparts;			/* the number of graph partitions */
	p_strat strategy;	/* partition strategy */
	bool dryrun;		/* true to disable running the consistency checking */
	bool threshold;
	bool pconly;		/* true to run only path-consistency */
	bool sanity_check;	/* true to check the correctness of the algorithm
						   (it is based on some invariable conditions that should
						    hold when the network is (path-)consistent) */
	bool mem_check;		/* true to estimate memory requirements (usually under-estimation) and exit */
	bool obj_volume;	/* true to partition using as the objective function communication volume */
	bool repartition;	/* true to re-partition the network after path-consistency has been executed */
	bool quiet;			/* true to print only statistical information when an input network has been processed */
	bool print_pgraph;	/* true to generate the computed partitioning graph to a file */
	netviz_frmt pgraph_format; /* the network file format to generate for visualizing the partitioning graph */

	cmd_options() {
		nparts 	  		= 0;
		strategy  		= BALANCED;
		dryrun 	  		= false;
		threshold 	  	= false;
		pconly 	 	 	= false;
		sanity_check 	= false;
		mem_check 	  	= false;
		obj_volume   	= false;
		repartition 	= false;
		quiet 			= false;
		print_pgraph    = false;
		pgraph_format   = NETVIZ_FRMT_END;
	}
} cmd_options;

#endif /* PRCC8_H_ */
